/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import java.lang.String;
import java.lang.StringBuffer;
import java.io.*;

 /* Instantiator of subcircuits (as thought by Anish on 22nd Jan 99) */

/**
 * Given a table of subckt definitions, s2iCktInstatiator creates instance
 * calls to the subckts in a second table. Names of elements and nodes are
 * made unique.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */

public class s2iCktInstantiator implements java.awt.event.ActionListener{

/**
 * The table containing the subckt definitions
 */
JTable from;

/**
 * The table containing the subckt instances
 */
JTable to;

/**
 * Construct an s2iCktInstantiator, with the tables and button specified.
 *
 * @author Anish Malhotra, Cadence Design Systems
 * @param from The JTable containing the subckt definitions.
 * @param to The JTable that will contain the subckt instantiations.
 * @param actionSource the button that triggers instantiations.
 */

  public s2iCktInstantiator(JTable from, JTable to, javax.swing.AbstractButton actionSource)
  {
     this.from = from;
     this.to = to;
     actionSource.addActionListener(this);
  }

/**
 * Create instance calls in the "to" to the selected subckts in the "from" table.
 *
 * @param event The event that triggered the action.
 */

  public void actionPerformed(java.awt.event.ActionEvent event)
  {
     if (event.getActionCommand().equals("Instantiate"))
     {
        // Get the number of rows to be instantiated
        int[] selections = from.getSelectedRows();
        int nRows = selections.length;
        if (nRows == 0) return ;

        // Get integers which are yet not suffixed in the Instantiated table
        String[] unique_sfx = getUniqueSuffix(to, nRows);

        // Prepare the list of strings corresponding to the instantiations
        javax.swing.table.TableModel dataModel =  from.getModel();
        int nCols = from.getColumnCount();
        java.lang.String newInstantiations[] = new String[nRows];
        java.lang.StringBuffer newStr = new StringBuffer();
        Object value;
        for (int i=0; i < nRows; i++, newStr.setLength(0))
        {
           value = dataModel.getValueAt(selections[i], 0);
           if (value != null)
           {
              newStr.append((String) value);
              newStr.append("  ");
           }
           for (int j=0; j < nCols; j++)
           {
              value = dataModel.getValueAt(selections[i], j);
              if (value != null)
              {
                 newStr.append((String) value);
                 newStr.append("_" + unique_sfx[i] + "  ");
              }
           }
           newInstantiations[i] = newStr.toString();
         }
    
        // Add the instantiation statements to the "to" table.
        for (int i=0; i < nRows; i++, newStr.setLength(0))
        {
           String Msg = newInstantiations[i];
           s2iError.Debug(Msg);
           ((s2iInstCktTableModel) to.getModel()).addInstance(newInstantiations[i]);
        }

        // Inform the "to" table about the changes.
        to.tableChanged(new TableModelEvent(to.getModel(), TableModelEvent.HEADER_ROW));

     }
  }

/**
 * Determine a suffix number that can be appended to all items in a table
 * row, such that uniqueness of the names is guaranteed.
 *
 * @param lookAt The JTable that must have unique rows.
 * @param num The number of the row that is to receive the unique suffix.
 * @return The unique suffix as a String
 */

  private String[] getUniqueSuffix(JTable lookAt, int num)
  {
     javax.swing.table.TableModel dataModel = lookAt.getModel();

     int nCols = lookAt.getColumnCount();
     int nRows = lookAt.getRowCount();

     // Determine the present integer suffixes.
     Hashtable intSuffixes = new Hashtable();
     String suffix;
     Object value;
     for (int i=0; i < nRows; i++)
     {
        for (int j=0; j < nCols; j++)
        {
           value = dataModel.getValueAt(i, j);
           if (value != null)
           {
              suffix = getIntSuffix((String) dataModel.getValueAt(i,j));
              intSuffixes.put(suffix, new Boolean(null));
           }
        }
     }

     // Now find suffixes which haven't been used yet.
     Vector freeSuffixes = new Vector();
     int found =0;
     for (int ctr =0 ; found < num; ctr++)
     {
       if (!intSuffixes.containsKey(Integer.toString(ctr)))
       {
          freeSuffixes.addElement(Integer.toString(ctr));
          found++; 
       }

       if (ctr == java.lang.Integer.MAX_VALUE) 
       {
          // TOO MANY NODES INSTANTIATED - CAN NOT DETERMINE UNIQUE SUFFIXES
       }

     }

     String[] freeSfx = new String[num];
     freeSuffixes.copyInto(freeSfx);
     return(freeSfx);

  }

/**
 * Determine the portion of a string that forms a trailing integer.
 *
 * @return The integer suffix as a String
 */

  private String getIntSuffix(String str)
  {
     char[] charSet = str.toCharArray();
     String suffix;
     int len = charSet.length;
     int i = len-1;

     for (i=len-1; ((i >= 0) && java.lang.Character.isDigit(charSet[i])) ; i--);

     i++;
     suffix = (str.substring(i));

     return(suffix);
   }

}
