/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import javax.swing.*;
import javax.swing.table.TableColumn;
import java.awt.*;
import javax.swing.JDialog;

import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JTable;

/**
 * A dialog that allows the user to configure the top-level subckt terminals
 * that are unconnected, to connect each one to the proper test fixture.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */

public class s2iConfigureNodes extends javax.swing.JDialog implements s2iDBOpenListener
{
/**
 * Default constructor.
 */

	public s2iConfigureNodes()
	{
		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		setTitle("Node/Net Configuration");
		setModal(true);
		getContentPane().setLayout(new BorderLayout(0,0));
		setVisible(false);
		setSize(413,304);
		jScrollPane1 = new javax.swing.JScrollPane();
		jScrollPane1.setBounds(0,0,413,279);
		jScrollPane1.setFont(new Font("Dialog", Font.PLAIN, 12));
		jScrollPane1.setForeground(new Color(0));
		getContentPane().add("Center", jScrollPane1);
		jConfigTable = new javax.swing.JTable();
		jConfigTable.setSelectionForeground(new java.awt.Color(16777215));
		jConfigTable.setSelectionBackground(new java.awt.Color(160));
		jConfigTable.setBounds(0,0,20,40);
		jConfigTable.setFont(new Font("Dialog", Font.PLAIN, 12));
		jConfigTable.setForeground(new Color(0));
		jConfigTable.setBackground(new Color(16777215));
		jScrollPane1.getViewport().add(jConfigTable);
		jPanel1 = new javax.swing.JPanel();
		jPanel1.setLayout(new BoxLayout(jPanel1,BoxLayout.X_AXIS));
		jPanel1.setBounds(0,279,413,25);
		jPanel1.setFont(new Font("Dialog", Font.PLAIN, 12));
		jPanel1.setForeground(new Color(0));
		getContentPane().add("South", jPanel1);
		jOK = new javax.swing.JButton();
		jOK.setText("  OK  ");
		jOK.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		jOK.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jOK.setBounds(0,0,63,25);
		jOK.setFont(new Font("Dialog", Font.PLAIN, 12));
		jOK.setForeground(new Color(0));
		jPanel1.add(jOK);
		jCancel = new javax.swing.JButton();
		jCancel.setText("Cancel");
		jCancel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jCancel.setBounds(63,0,73,25);
		jCancel.setFont(new Font("Dialog", Font.PLAIN, 12));
		jCancel.setForeground(new Color(0));
		jPanel1.add(jCancel);
		//}}


        //{{ Non-symantec init controls - anish

        //}}


		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		jOK.addActionListener(lSymAction);
		jCancel.addActionListener(lSymAction);
		//}}
	}

/**
 * Constructor with window title setting.
 *
 * @param sTitle The window title String.
 */
	public s2iConfigureNodes(String sTitle)
	{
		this();
		setTitle(sTitle);
	}

/**
 * Constructor with window node preloading.
 *
 * @param nodes The subckt nodes to show in the table.
 */
	public s2iConfigureNodes(java.util.Enumeration nodes)
	{
	    this();
	    this.nets = nodes;

	    //Add the nodes/nets to the Configuration Table Model
	    conTableModel = new s2iConfigTableModel();
	    conTableModel.updateData(nodes);
	    conTableModel.mark();
        jConfigTable.setModel(conTableModel);

        // Put the renderer for the pin-type
        {
            TableColumn pinColumn = jConfigTable.getColumnModel().getColumn(1);
            JComboBox comboBox = new JComboBox();
            comboBox.addItem("Input");
            comboBox.addItem("Output");
            comboBox.addItem("Vcc");
            comboBox.addItem("Gnd");
            comboBox.addItem("Enable");
            comboBox.addItem("Float");
            comboBox.addItem("I/O");
            comboBox.addItem("");
            pinColumn.setCellEditor( new DefaultCellEditor(comboBox));
        }
	}

/**
 * Set window visibility. This also relocates the window and takes care of
 * data synchronization.
 *
 * @param b true to make the window visible.
 */
	public void setVisible(boolean b)
	{
		if (b)
			setLocation(50, 50);
		super.setVisible(b);
		/* Non-symantec code
		 * Get the values from the DB; and register as DB open listener
		 * Stay DB open listener till this form is alive
		 */
		if (b)
		{
		   getDB();
		   s2iDBReadWrite.addDBOpenListener(this);
		}
		else
		{
		   s2iDBReadWrite.removeDBOpenListener(this);
		}
	}

/**
 * Main routine for testing.
 *
 @param arg unused
 */
	static public void main(String args[])
	{
		(new s2iConfigureNodes()).setVisible(true);
	}

 /**
 * Call the superclass addNotify(), then readjust the layout.
 */

	public void addNotify()
	{
		// Record the size of the window prior to calling parents addNotify.
		Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;
		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getContentPane().getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

	/**
   * Used for addNotify check.
   */
	boolean fComponentsAdjusted = false;

	//{{DECLARE_CONTROLS
	javax.swing.JScrollPane jScrollPane1;
	javax.swing.JTable jConfigTable;
	javax.swing.JPanel jPanel1;
	javax.swing.JButton jOK;
	javax.swing.JButton jCancel;
	//}}

	//{{Declaring non-symantec controls - anish
	s2iConfigTableModel conTableModel;
	java.util.Enumeration nets;
	//}}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == s2iConfigureNodes.this)
				JDialog1_WindowClosing(event);
		}
	}

/**
 * Handle window closing.
 *
 @param event unused
 */
	void JDialog1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false); // hide the Frame
		dispose();	     // free the system resources
		//System.exit(0);    // close the application
	}

/**
 * Main routine for testing.
 *
 @return The JTable object.
 */
	public JTable getConfigTable()
	{
	    return (jConfigTable);
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == jOK)
				jOK_actionPerformed(event);
			else if (object == jCancel)
				jCancel_actionPerformed(event);
		}
	}

/**
 * Handle OK button clicks. The table data is saved and the window closed.
 *
 @param event unused
 */
	void jOK_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		// Archive the configuration i.e.. Save to the main DB
		s2iGlobals.gConfigVector = conTableModel.archive();

		// Create a Window closing event and dispatch it to this.
           java.awt.AWTEvent  e = new java.awt.event.WindowEvent(this, java.awt.event.WindowEvent.WINDOW_CLOSING);
           dispatchEvent(e);
	}

/**
 * Handle OK button clicks. The table data is restored and the window closed.
 *
 @param event unused
 */
	void jCancel_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		((s2iConfigTableModel) jConfigTable.getModel()).restore();
		// Create a Window closing event and dispatch it to this.
        java.awt.AWTEvent  e = new java.awt.event.WindowEvent(this, java.awt.event.WindowEvent.WINDOW_CLOSING);
        dispatchEvent(e);
	}

 /**
  * Clear the table data and reload from the project data.
  *
  * @see s2iConfigTableModel
  */
	public void getDB()
	{
	   final java.util.Vector emptyVector = new java.util.Vector();

	   ((s2iConfigTableModel) jConfigTable.getModel()).
	       updateData(emptyVector.elements());
	   ((s2iConfigTableModel) jConfigTable.getModel()).
	       updateData(s2iGlobals.gConfigVector.elements());
	}
}
