/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * A class for reading Spice library files, which contain .LIB and .MODEL
 * keywords.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */
public class s2iLibReader extends s2iSpiceReader
{
/**
 * Construct a LibReader that reads a file and returns info
 * @param inFile The Spice file to read
 */
   public s2iLibReader(String inFile)
       throws java.io.FileNotFoundException
   {
       super(inFile);
   }
/**
 * Construct a LibReader that reads a file and writes to another file.
 * @param inFile The Spice file to read
 * @param outFile The Spice file to write
 */
   public s2iLibReader(String inFile, String outFile)
       throws java.io.FileNotFoundException, java.io.IOException
   {
       super(inFile, outFile);
   }
/**
 * Get the list of names appearing on Spice .MODEL lines.
 * @return The Enumeration of model names
 * @exception java.io.IOException
 */
   public Enumeration getModelNames() throws java.io.IOException
   {
      Vector ModelNames = new Vector();
      StringTokenizer tokens;
      String word, name;
      StringBuffer SpiceLine = new StringBuffer();
      Vector dummy = new Vector();

      while( getSpiceLine(SpiceLine, dummy) != 0)
      {
        //Tokenize and look for a line starting with .Model
        tokens = new StringTokenizer(SpiceLine.toString());
        if (tokens.hasMoreTokens())
        {
           word = tokens.nextToken();
           if (word.equalsIgnoreCase(s2iGlobals.KWS_MODEL))
           {
              if (tokens.hasMoreTokens())
                 name = tokens.nextToken();
              else
                 name = null;
              if (name != null)
                 ModelNames.addElement(name);
              if (buf_writer != null)
              {
                 buf_writer.write(name);
                 buf_writer.newLine();
              }
           }
        }

      }
      close();
      return(ModelNames.elements());
   }

/**
 * Get the list of file names appearing on Spice .LIB lines.
 * @return The Enumeration of file names
 * @exception java.io.IOException
 */
   public Enumeration getEntryNames() throws java.io.IOException
   {
      Vector EntryNames = new Vector();
      StringTokenizer tokens;
      String word, name;
      StringBuffer SpiceLine = new StringBuffer();
      Vector dummy = new Vector();

      while( getSpiceLine(SpiceLine, dummy) != 0)
      {
        //Tokenize and look for a line starting with .LIB
        tokens = new StringTokenizer(SpiceLine.toString());
        if (tokens.hasMoreTokens())
        {
           word = tokens.nextToken();
           if (word.equalsIgnoreCase(s2iGlobals.KWS_LIB))
           {
              if (tokens.hasMoreTokens())
                 name = tokens.nextToken();
              else
                 name = null;
              if (name != null)
                 EntryNames.addElement(name);
              if (buf_writer != null)
              {
                 buf_writer.write(name);
                 buf_writer.newLine();
              }
           }
        }

      }
      close();
      return(EntryNames.elements());
   }

/**
 * A main method for testing this class.
 * Usage: s2iLibReader <SPICE filename> [<output FileName>]
 * @param args 1 or 2 file names are required
 */
   public static void main(String args[])
   {
      s2iLibReader reader =null;
      Enumeration names = null;
      String in_file, out_file;

      if ((args.length != 1) && (args.length != 2))
      {
         System.err.println("Usage: s2iLibReader <SPICE filename> [<output FileName>]");
         System.exit(0);
      }

      in_file = args[0];
      try
      {
         if (args.length == 1)
         {
             reader = new s2iLibReader(in_file);
         }
         else
         {
             out_file = args[1];
             reader = new s2iLibReader(in_file, out_file);
         }
      }
      catch (java.io.FileNotFoundException FileNotFnd)
      {
         s2iError.PrintMsg("SPICE Library not found: " + FileNotFnd.getMessage(),
                s2iError.ERR, s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                null);
      }
      catch (java.io.IOException IOex)
      {
         s2iError.PrintMsg(IOex.toString() + IOex.getMessage(),
                s2iError.ERR, s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                null);
      }

      try
      {
         if (reader !=null) names = reader.getModelNames();
      }
      catch (java.io.IOException ex)
      {
         System.err.println("I/O error while reading SPICE Library-" + in_file);
         s2iError.PrintMsg("I/O error while reading SPICE Library-" + in_file,
                s2iError.ERR, s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                null);
      }
      if (names !=  null)
         while (names.hasMoreElements())
            System.out.println( (String) names.nextElement());
   }

}
