/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */



/* Anish - 17 Nov 98
 *
 */

import java.util.Properties;

public class s2iProperties{
private static java.util.Properties prop=null; 
private static java.io.OutputStream outStr;

   public static void loadProperties(boolean clear)
   {
      String fullpath;
      fullpath  = (s2iGlobals.ProjDirFile == null) ? s2iGlobals.propFileName : s2iGlobals.ProjDirFile + s2iGlobals.propFileName;
      loadProperties(clear, fullpath);
   }


   public static void loadProperties(boolean clear, String fileName)
   {
      java.io.FileInputStream inProp = null;
      if (fileName == null)
         fileName  = (s2iGlobals.ProjDirFile == null) ? s2iGlobals.propFileName : s2iGlobals.ProjDirFile + s2iGlobals.propFileName;

      /* Each time a NON-new property is loaded the present one is
       * made the default. A default may already be having its own
       * default. This way an arbitrarily deep chain of property 
       * lists can be obtained.
       */
      prop = (clear) ? new java.util.Properties(null) : new java.util.Properties(prop);
      try{
         inProp = new java.io.FileInputStream(fileName);
         prop.load(inProp);
      } catch (java.io.FileNotFoundException Filex) {
         String Msg = "Property file \n" + Filex.getMessage() + " not found";
         s2iError.PrintMsg(Msg, s2iError.WARN, s2iError.LOG | s2iError.POPUP, null);
      } catch (java.io.IOException ioe) {
         String Msg = "Failed to load property file \n" + fileName;
         s2iError.PrintMsg(Msg, s2iError.WARN, s2iError.LOG | s2iError.POPUP, null);
      }
   }


   public static void loadDefaultProperties(boolean clear)
   {
      /* Each time a NON-new property is loaded the present one is
       * made the default. A default may already be having its own
       * default. This way an arbitrarily deep chain of property 
       * lists can be obtained.
       */
      if (clear) 
      {
         prop = new java.util.Properties(getDefaultProperties(null));
      }
      else
      {
         prop = new java.util.Properties(getDefaultProperties(prop));
      }
      
   }

   public static void saveProperties()
   {
      String fullpath  = (s2iGlobals.ProjDirFile == null) ? s2iGlobals.propFileName : s2iGlobals.ProjDirFile + s2iGlobals.propFileName;
      try{
         outStr = new java.io.FileOutputStream(fullpath);
         prop.store(outStr, null);
      }
      catch(java.io.IOException ioe)
      {
         s2iError.PrintMsg("Failed to save properties", s2iError.ERR,
                           s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                           null);
      }
   }

  private static java.util.Properties getDefaultProperties(Properties moreDefaults)
  {
     Properties property = new Properties(moreDefaults);
     String os = System.getProperty("os.name");
     if (os.equals("Solaris"))
     {
        property.put("EDITOR", s2iGlobals.DEF_SOLARIS_EDITOR );
     }
     else if (os.equals("HP-UX"))
     {
        property.put("EDITOR", s2iGlobals.DEF_HP_EDITOR );
     }
     else if (os.equals("Windows NT"))
     {
        property.put("EDITOR",  s2iGlobals.DEF_WIN_EDITOR);
     }
     else if (os.equals("AIX"))
     {
        property.put("EDITOR",  s2iGlobals.DEF_IBM_EDITOR);
     }

     return(property);
  }

  /* This function will get the editor command depending on
   * the EDITOR setting in the property file and the platform.
   */
  public static String getEditorCommand()
  {
     String command = null;
     String editor = prop.getProperty("EDITOR");
     if (editor != null)
     {
        String os = System.getProperty("os.name");
        s2iError.Debug("OS=" + os);
     }
     s2iError.Debug("EDITOR=" + editor);
     return(editor);
  }

  public static void main(String arg[])
  {
     loadDefaultProperties(true);
     getEditorCommand();
     saveProperties();
  }


} // End of Class
