/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */



import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.lang.String;
import java.util.StringTokenizer;
import java.awt.List;

public class s2iReader
{
   /**
    * set to true if the current token is in model section
    */  
   boolean modelProc = false;
   /**
    * set to true if the current token is in component section
    */  
   boolean compProc = false;
   /**
    * set to true if the current token is in global param section
    */  
   boolean globalProc = false;

   /**  
    * The constructor opens the given .s2i file, parses it, and populates
    * the given s2i data structure.
    *   
    * @param inFile         input .s2i file
    * @param sp2i           main object containing s2i data structure
    * @see                  StringTokenizer
    */  
   public void Sp2iParse( String inFile, s2iData fileData)
   {
      BufferedReader inBufRdr;
      String  line;
      String key = null;

      if ((inFile!=null) && (!inFile.trim().equals("")))
      {
         //If the input fileName is specified.

         // initialise the data structure
         // initS2i(fileData, true);
 
         try
         {
	    int pinNo = 0;
	    inBufRdr = new BufferedReader(new InputStreamReader(new
			   FileInputStream(inFile)));
            line = inBufRdr.readLine();
	    while (line != null)
	    {
	       char firstChar = ' ';
	       if (line.trim().length() > 0)
               {
		  firstChar = line.trim().charAt(0);
               }
               else
               {
                  line = inBufRdr.readLine();
                  continue;
               }
	       if(firstChar == '[')
	       {
		  // word to the left of ']' is key, to the right is argument
	          StringTokenizer st1 = new StringTokenizer(line, "]");
	          int numToken1 = st1.countTokens();
		  if (numToken1 != 0)
		  {
		     StringTokenizer st2 = new StringTokenizer(st1.nextToken(), "[");
		     int numToken2 = st2.countTokens();
		     if (numToken2 != 0)
		     {
			key = st2.nextToken();
			String arg = (numToken1 > 1) ? st1.nextToken().trim() : new String("");
			// now the further parsing based on key value
			processKey( key, arg, fileData);
		     }
		  }
	       }
	       else if( firstChar == '|')
	       {
		  //comments are to be ignored
	       }
	       else
	       {
		  //if the last key was 'Pin'
                  if ((key != null) && (key.equalsIgnoreCase("Pin")))
		  {
                     StringTokenizer st3 = new StringTokenizer( line );
		     String firstToken = " ";
                     int cnt3 = st3.countTokens();
                     if( cnt3 != 0 )
                     {
                        firstToken = st3.nextToken();
                     }

		     if ((firstToken != null) && (firstToken.equalsIgnoreCase("->")))
		     {
			if (cnt3==2)
			{
			   fileData.input_pin =	st3.nextToken();
			}
			else if (cnt3==3)
			{
			   fileData.input_pin =	 st3.nextToken();
			   fileData.enable_pin = st3.nextToken();
			}
		     }
		     else if (cnt3==4)
		     {
		   	String ignoreThis;
			if (line.trim().endsWith("POWER"))
                        {
                           fileData.pow_pin = st3.nextToken();
                        }
			else if (line.trim().endsWith("GND"))
                        {
                           fileData.gnd_pin = st3.nextToken();
                        }
			else if (line.trim().endsWith("none"))
                        {
                           // Must be enable and input node. Ignore it
                           // We'll be recording these after the "->"
                        }
			else 
                        {
			   fileData.spice_node = st3.nextToken();
		   	   ignoreThis = st3.nextToken();
			   fileData.model_name = st3.nextToken();
                        }
		     }
		     else
		     {
                        /* **************************************************
                        ** A 'PIN' statement not starting with "->" and not *
                        ** having 4 words is an erroneous statement as per  *
                        ** s2i command file syntax. Just ignore it.         *
                        * **************************************************/
		     }
		  }
	       }
               line = inBufRdr.readLine();
	    }
         }
	 catch(java.io.IOException ioe)
	 {
	    String Msg = ioe.getMessage();
	    s2iError.PrintMsg(Msg, s2iError.ERR,
                          s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                          null);
	 }
      }
   }


   String newVal(String args)
   {
      String str;
      if((args == null) || (args.equals("")) || (args.equals("NA")))
      {
         str = new String("");
      }
      else
      {
         str = new String(args);
      }
      return str;
   }


   /**
    * Depending on the key value, args are parsed and corresponding
    * s2i data structure is stuffed.
    *   
    * @param key            the key value denoting parameter type
    * @param args           parameters values
    * @param sp2i           object s2i data structure
    */  
   void processKey( String key, String args, s2iData sp2i )
   {
      String  firstArg = null;
      if( args != null )
      {
         StringTokenizer st = new StringTokenizer( args );
         int cnt = st.countTokens();
         if( cnt != 0 )
         {
            firstArg = st.nextToken();
         }
      }

      if( key.equalsIgnoreCase( "IBIS Ver" ) )
      {
         globalProc = true;
         compProc = false;
         modelProc = false;
         sp2i.ibsVersion = newVal(firstArg);
      }

      if( key.equalsIgnoreCase("File name"))
      {
         sp2i.ibsFileName = newVal(firstArg);
      }

      if( key.equalsIgnoreCase("File rev"))
      {
         sp2i.ibsFileRev = newVal(firstArg);
      }

      if( key.equalsIgnoreCase( "Date" ))
      {
         sp2i.ibsDate = newVal( args );
      }

      if( key.equalsIgnoreCase( "Source" ))
      {
         sp2i.ibsSource = newVal( args );
      }

      if( key.equalsIgnoreCase( "Notes" ))
      {
         sp2i.ibsNotes = new String( args );
      }

      if( key.equalsIgnoreCase( "Disclaimer" ))
      {
         sp2i.ibsDisclaimer = newVal( args );
      }

      if( key.equalsIgnoreCase( "Copyright" ))
      {
         sp2i.ibsCopyright = newVal( args );
      }

      if( key.equalsIgnoreCase( "Spice type" ) )
      {
         sp2i.ibsSpiceType = newVal( firstArg );
         if( firstArg.toUpperCase().equals( "HSPICE" ) )
         {
            sp2i.ibsSpiceType = new String( "HSpice" );
         }
         else if( firstArg.toUpperCase().equals( "PSPICE" ) )
         {
            sp2i.ibsSpiceType = new String( "PSpice" );
         }
         else if( firstArg.toUpperCase().equals( "SPICE2" ) )
         {
            sp2i.ibsSpiceType = new String( "Spice2" );
         }
         else if( firstArg.toUpperCase().equals( "SPICE3" ) )
         {
            sp2i.ibsSpiceType = new String( "Spice3" );
         }
         else if( firstArg.toUpperCase().equals( "SPECTRE" ) )
         {
            sp2i.ibsSpiceType = new String( "Spectre" );
         }
      }
 
      if( key.equalsIgnoreCase( "Spice command" ) )
      {
         sp2i.ibsSpiceComm = newVal( args );
      }
 
      if( key.equalsIgnoreCase( "Iterate" ) )
      {
         sp2i.ibsIterate = true;
      }
 
      if( key.equalsIgnoreCase( "Cleanup" ) )
      {
         sp2i.ibsCleanup = true;
      }
 
      if( key.equalsIgnoreCase( "Summarize" ) )
      {
         sp2i.ibsSummarize = true;
      }

      /* **************************************************************
       * Some more code to come here
       *****************************************************************/

      if (key.equalsIgnoreCase( "Temperature range" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsTempTtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsTempTmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsTempTmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Voltage range" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsVoltVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsVoltVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsVoltVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Pullup reference" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsPullupVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsPullupVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsPullupVmax = newVal( temp );
               }
            }
         }
      }
      if( key.equalsIgnoreCase( "Pulldown reference" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsPulldnVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsPulldnVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsPulldnVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "POWER clamp reference" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsPclampVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsPclampVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsPclampVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "GND clamp reference" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsGclampVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsGclampVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsGclampVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "R_pkg" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsRpkgtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsRpkgmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsRpkgmax = newVal( temp );
               }
            }
         }
      }
      if( key.equalsIgnoreCase( "L_pkg" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsLpkgtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsLpkgmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsLpkgmax = newVal( temp );
               }
            }
         }
      }
      if( key.equalsIgnoreCase( "C_pkg" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsCpkgtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsCpkgmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsCpkgmax = newVal( temp );
               }
            }
         }    
      }
      if( key.equalsIgnoreCase( "C_comp" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsCcompCtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsCcompCmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsCcompCmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Rload" ))
      {
         sp2i.ibsRload = newVal( firstArg );
      }
      if (key.equalsIgnoreCase( "Sim time" ))
      {
         sp2i.ibsSimTime = newVal( firstArg );
      }
      if (key.equalsIgnoreCase( "Vil" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsVilVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsVilVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsVilVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Vih" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsVihVtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsVihVmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsVihVmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Tr" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsTrTtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsTrTmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsTrTmax = newVal( temp );
               }
            }
         }
      }
      if (key.equalsIgnoreCase( "Tf" ))
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsTfTtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsTfTmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsTfTmax = newVal( temp );
               }
            }
         }
      }
      if( key.equalsIgnoreCase( "Clamp tolerance" ) )
      {
         sp2i.ibsClampTol = newVal( firstArg );
      }
      if( key.equalsIgnoreCase( "Derate VI" ) )
      {
         sp2i.ibsDerateVi = newVal( firstArg );
      }
      if( key.equalsIgnoreCase( "Derate ramp" ) )
      {
         sp2i.ibsDerateRamp = newVal( firstArg );
      }


      /* **************************************************************
       * Some more code to come till here
       *****************************************************************/


      if (key.equalsIgnoreCase( "Component" ))
      {
         sp2i.ibsComponent = newVal( firstArg );
      }

      if (key.equalsIgnoreCase( "Manufacturer" ))
      {
         sp2i.ibsManufacturer = newVal( args );
      }
 
      if (key.equalsIgnoreCase( "Package model" ))
      {
         sp2i.ibsPackageName = newVal( args );
      }
 
      if (key.equalsIgnoreCase( "Spice file" ))
      {
         sp2i.ibsSpiceFile = newVal( firstArg );
      }
      if (key.equalsIgnoreCase( "Model" ))
      {

      }
      if( key.equalsIgnoreCase( "NoModel" ) )
      {
         sp2i.ibsNoModel = newVal( "Yes" );
      }
      if( key.equalsIgnoreCase( "Model type" ) )
      {
         sp2i.ibsModelType = newVal( firstArg );
         if( firstArg.toUpperCase().equals( "INPUT" ) )
         {
            sp2i.ibsModelType = new String( "Input" );
         }
         else if( firstArg.toUpperCase().equals( "OUTPUT" ) )
         {
            sp2i.ibsModelType = new String( "Output" );
         }
         else if( firstArg.toUpperCase().equals( "I/O" ) )
         {
            sp2i.ibsModelType = new String( "I/O" );
         }
         else if( firstArg.toUpperCase().equals( "3-STATE" ) )
         {
            sp2i.ibsModelType = new String( "3-State" );
         }
         else if( firstArg.toUpperCase().equals( "OPEN_DRAIN" ) )
         {
            sp2i.ibsModelType = new String( "Open_drain" );
         }
         else if( firstArg.toUpperCase().equals( "I/O_OPEN_DRAIN" ) )
         {
            sp2i.ibsModelType = new String( "I/O_open_drain" );
         }
         else if( firstArg.toUpperCase().equals( "OPEN_SINK" ) )
         {
            sp2i.ibsModelType = new String( "Open_sink" );
         }
         else if( firstArg.toUpperCase().equals( "I/O_OPEN_SINK" ) )
         {
            sp2i.ibsModelType = new String( "I/O_open_sink" );
         }
         else if( firstArg.toUpperCase().equals( "OPEN_SOURCE" ) )
         {
            sp2i.ibsModelType = new String( "Open_source" );
         }
         else if( firstArg.toUpperCase().equals( "I/O_OPEN_SOURCE" ) )
         {
            sp2i.ibsModelType = new String( "I/O_open_source" );
         }
         else if( firstArg.toUpperCase().equals( "INPUT_ECL" ) )
         {
            sp2i.ibsModelType = new String( "Input_ECL" );
         }
         else if( firstArg.toUpperCase().equals( "OUTPUT_ECL" ) )
         {
            sp2i.ibsModelType = new String( "Output_ECL" );
         }
         else if( firstArg.toUpperCase().equals( "I/O_ECL" ) )
         {
            sp2i.ibsModelType = new String( "I/O_ECL" );
         }
         else if( firstArg.toUpperCase().equals( "TERMINATOR" ) )
         {
            sp2i.ibsModelType = new String( "Terminator" );
         }
      }

      if( key.equalsIgnoreCase( "Polarity" ) )
      {
         sp2i.ibsPolarity = newVal( firstArg );
         if( firstArg.toUpperCase().equals( "NON-INVERTING" ) )
         {
                sp2i.ibsPolarity = new String( "Non-Inverting" );
         }
         else if( firstArg.toUpperCase().equals( "INVERTING" ) )
         {
                sp2i.ibsPolarity = new String( "Inverting" );
         }
      }

      if( key.equalsIgnoreCase( "Enable" ) )
      {
         sp2i.ibsEnable = newVal( firstArg );
         if( firstArg.toUpperCase().equals( "ACTIVE-HIGH" ) )
         {
            sp2i.ibsEnable = new String( "Active-High" );
         }
         else if( firstArg.toUpperCase().equals( "ACTIVE-LOW" ) )
         {
            sp2i.ibsEnable = new String( "Active-Low" );
         }
      }
 
      if( key.equalsIgnoreCase( "Vinl" ) )
      {
         sp2i.ibsVin = newVal( firstArg );
      }
 
      if( key.equalsIgnoreCase( "Vinh" ) )
      {
         sp2i.ibsVinh = newVal( firstArg );
      }
 
      if( key.equalsIgnoreCase( "Vmeas" ) )
      {
         sp2i.ibsVmeas = newVal( firstArg );
      }

      if( key.equalsIgnoreCase( "Cref" ) )
      {
         sp2i.ibsCref = newVal( firstArg );
      }
 
      if( key.equalsIgnoreCase( "Rref" ) )
      {
         sp2i.ibsRref = newVal( firstArg );
      }
 
      if( key.equalsIgnoreCase( "Vref" ) )
      {
         sp2i.ibsVref = newVal( firstArg );
      }

      if( key.equalsIgnoreCase( "Rgnd" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsRgndRtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsRgndRmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsRgndRmax = newVal( temp );
               }
            }
         }
      }
 
      if( key.equalsIgnoreCase( "Rpower" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsRpowerRtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsRpowerRmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsRpowerRmax = newVal( temp );
               }
            }
         }
      }

      if( key.equalsIgnoreCase( "Rac" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsRacRtyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsRacRmin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsRacRmax = newVal( temp );
               }
            }
         }
      }
 
      if( key.equalsIgnoreCase( "Cac" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                        sp2i.ibsCacCtyp = newVal( temp );
               }
               if(i==1)
               {
                        sp2i.ibsCacCmin = newVal( temp );
               }
               if(i==2)
               {
                        sp2i.ibsCacCmax = newVal( temp );
               }
            }
         }
      }

      if( key.equalsIgnoreCase( "Model file" ) )
      {
         if( args != null )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsModelFiletyp = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsModelFilemin = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsModelFilemax = newVal( temp );
               }
            }
         }
      }

      if( key.equalsIgnoreCase( "Rising waveform" ) )
      {
         if(( args != null ) && (sp2i.RwaveCtr<2) )
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsRwaveRf[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsRwaveVf[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsRwaveVfMin[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==3)
               {
                  sp2i.ibsRwaveVfMax[sp2i.RwaveCtr]= newVal( temp );
               }
               if(i==4)
               {
                  sp2i.ibsRwaveLf[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==5)
               {
                  sp2i.ibsRwaveCf[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==6)
               {
                  sp2i.ibsRwaveRd[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==7)
               {
                  sp2i.ibsRwaveCd[sp2i.RwaveCtr] = newVal( temp );
               }
               if(i==8)
               {
                  sp2i.ibsRwaveLd[sp2i.RwaveCtr] = newVal( temp );
               }
            }
            sp2i.RwaveCtr++;
         }    
      }

      if( key.equalsIgnoreCase( "Falling waveform" ) )
      {
         if(( args != null ) && (sp2i.FwaveCtr < 2))
         {
            StringTokenizer st = new StringTokenizer( args );
            int cnt = st.countTokens();
            for(int i=0; i<cnt; ++i)
            {
               String temp = st.nextToken();
               if(i==0)
               {
                  sp2i.ibsFwaveRf[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==1)
               {
                  sp2i.ibsFwaveVf[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==2)
               {
                  sp2i.ibsFwaveVfMin[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==3)
               {
                  sp2i.ibsFwaveVfMax[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==4)
               {
                  sp2i.ibsFwaveLf[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==5)
               {
                  sp2i.ibsFwaveCf[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==6)
               {
                  sp2i.ibsFwaveRd[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==7)
               {
                  sp2i.ibsFwaveCd[sp2i.FwaveCtr] = newVal( temp );
               }
               if(i==8)
               {
                  sp2i.ibsFwaveLd[sp2i.FwaveCtr] = newVal( temp );
               }
            }
            sp2i.FwaveCtr++;
         }    
      }
   }


   public static void main(String args[])
   {
      s2iData data = new s2iData();
      s2iReader Rdr = new s2iReader();
      Rdr.Sp2iParse(args[0], data);

   }

}
