/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */



import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.util.StringTokenizer;

public class s2iWriter
{

   /**
   * This function verifies that a given string is non-empty
   *
   * @param str            the string to be verified
   */
   private boolean nonEmpty(String str)
   {
      if ((str == null) || (str.trim().equals("")))
      {
         return false;
      }
      else
      {
         return true;
      }
   }


   /**
   * This function returns "NA" as the value for an empty string
   * else the string is returned.
   *
   * @param str            the string to be evaluated
   * @return str           non-empty string returned
   */
   private String getVal(String str)
   {
      if (nonEmpty(str))
      {
         return(str);
      }
      else
      {
         return("NA");
      }
   }


   public void  outputS2i(String fileName, s2iData data, boolean putCmt)
      throws java.io.IOException
   {
      /* ****************************
       * Open the file for output   *
       ******************************/ 
      PrintWriter outStream = new PrintWriter(new FileOutputStream(fileName));

      /* **********************************
       * Output the data to the s2i file  *
       ************************************/ 
      if (putCmt)
      {
	 outStream.println("| THE HEADER");
	 outStream.println("|   IBIS Version = 1.1 or 2.1" );
      }
      outStream.println("[IBIS Ver] " + " " + getVal(data.ibsVersion));
      outStream.println("");
      if (nonEmpty(data.ibsFileName))
      {
	 outStream.println("[File name]" + " " + getVal(data.ibsFileName));
         outStream.println("");
      }
      else
      {
	 if (putCmt) outStream.println("|[File name] " );
      }
      if (putCmt) 
         outStream.println( "|   This describes the current file revision." );
      outStream.println( "[File rev] " + " " + getVal(data.ibsFileRev) );
      outStream.println( "" );

      if (putCmt) outStream.println( "|   File date" );
      outStream.println( "[Date] " + " " + getVal(data.ibsDate) );
      outStream.println( "" );

      if (putCmt) outStream.println( "|   File source" );
      outStream.println( "[Source] " + " " + getVal(data.ibsSource) );
      outStream.println( "" );

      if (putCmt) outStream.println( "|   File notes" );
      outStream.println( "[Notes] " + " " + getVal(data.ibsNotes) );
      outStream.println("");

      if (putCmt) outStream.println( "|   File disclaimer" );
      outStream.println( "[Disclaimer] " + " " + getVal(data.ibsDisclaimer) );
      outStream.println("");
 
      if (putCmt) outStream.println( "|   File copyright" );
      outStream.println( "[Copyright] " + " " + getVal(data.ibsCopyright) );
      outStream.println("");
 
      if (putCmt) 
      {
         outStream.println( "|   Describes which flavor of Spice to use when simulating the circuits." );
         outStream.println( "|   Possible types HSpice, PSpice, Spice2, Spice3, or Spectre" );
      }
      outStream.println( "[Spice type] " + " " + getVal(data.ibsSpiceType) );
      outStream.println("");
 
      if (putCmt) 
      {
         outStream.println( "|   This string specifies, in C syntax, how you would call Spice" );
         outStream.println( "|   from the command line." );
      }
      if (nonEmpty(data.ibsSpiceComm))
      {
         outStream.println( "[Spice command] " + " " + getVal(data.ibsSpiceComm) );
         outStream.println("");
      }

      if (putCmt) outStream.println( "|   Iterate flag" );
      if (data.ibsIterate)
      {
         outStream.println( "[Iterate] " );
         outStream.println( "" );
      }
      else
      {
         if (putCmt) outStream.println("|[Iterate] ");
      }
      outStream.println("");
 
      if (putCmt) 
      {
         outStream.println("|   When this command is specified, s2ibis2 will delete all of the");
         outStream.println("|   spice input, output and message files as it proceeds.");
      }
      if (data.ibsCleanup)
      {
         outStream.println("[Cleanup] ");
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[Cleanup] " );
      }
 
      if (putCmt) 
      {
         outStream.println("|   When this command is specified, s2ibis2 will summarize all of");
         outStream.println("|   the component data and ask if it's OK to proceed with the");
         outStream.println("|   component analysis.");
      }
      if (data.ibsSummarize)
      {
         outStream.println("[Summarize] ");
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Summarize] ");
      }

      if (putCmt)
      {
         outStream.println("|   Specifies the temperature, in degrees C, under which to run the");
         outStream.println("|   TYP, MIN and MAX simulations." );
      }
      if (nonEmpty(data.ibsTempTtyp))
      {
         outStream.println("[Temperature range] " 
                            + " " + getVal(data.ibsTempTtyp)
                            + " " + getVal(data.ibsTempTmin)
                            + " " + getVal(data.ibsTempTmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Temperature range] typ min max");
      }


      if (putCmt)
      {  
         outStream.println("|   Specifies the supply rail voltage." );
         outStream.println("|   NOTE:  You MUST specify a voltage range for all models, either" );
         outStream.println("|   with this command or with ALL FOUR of the following commands." );
      }
      if (nonEmpty(data.ibsVoltVtyp))
      { 
         outStream.println("[Voltage range] " 
                            + " " + getVal(data.ibsVoltVtyp)
                            + " " + getVal(data.ibsVoltVmin)
                            + " " + getVal(data.ibsVoltVmax));
         outStream.println(""); 
      } 
      else 
      { 
         if (putCmt) outStream.println("|[Voltage range] typ min max"); 
      }


      if (putCmt) outStream.println("|Specifies the pullup reference supply.");
      if (nonEmpty(data.ibsPullupVtyp))
      {
         outStream.println("[Pullup reference] " 
                            + " " + getVal(data.ibsPullupVtyp)
                            + " " + getVal(data.ibsPullupVmin)
                            + " " + getVal(data.ibsPullupVmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[Pullup reference] typ min max" );
      }


      if (putCmt) outStream.println("|Specifies the pulldown reference supply.");
      if (nonEmpty(data.ibsPulldnVtyp))
      {
         outStream.println("[Pulldown reference] " 
                            + " " + getVal(data.ibsPulldnVtyp)
                            + " " + getVal(data.ibsPulldnVmin)
                            + " " + getVal(data.ibsPulldnVmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[Pulldown reference] typ min max" );
      }


      if (putCmt) outStream.println("|Specifies the power clamp reference supply.");
      if (nonEmpty(data.ibsPclampVtyp))
      {
         outStream.println("[POWER clamp reference] " 
                            + " " + getVal(data.ibsPclampVtyp)
                            + " " + getVal(data.ibsPclampVmin)
                            + " " + getVal(data.ibsPclampVmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[POWER clamp reference] typ min max" );
      }


      if (putCmt) outStream.println("|Specifies the ground clamp reference supply.");
      if (nonEmpty(data.ibsGclampVtyp))
      {
         outStream.println("[GND clamp reference] "
                           + " " + getVal(data.ibsGclampVtyp)
                           + " " + getVal(data.ibsGclampVmin)
                           + " " + getVal(data.ibsGclampVmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[GND clamp reference] typ min max" );
      }


      if (putCmt) outStream.println("| Describes the default resistance.");
      if (nonEmpty(data.ibsRpkgtyp))
      {
         outStream.println("[R_pkg] "
                            + " " + getVal(data.ibsRpkgtyp)
                            + " " + getVal(data.ibsRpkgmin)
                            + " " + getVal(data.ibsRpkgmax)); 
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[R_pkg] typ min max" );
      }


      if (putCmt) outStream.println("| Describes the default inductance.");
      if (nonEmpty(data.ibsLpkgtyp))
      {
         outStream.println("[L_pkg] " 
                            + " " + getVal(data.ibsLpkgtyp)
                            + " " + getVal(data.ibsLpkgmin)
                            + " " + getVal(data.ibsLpkgmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[L_pkg] typ min max" );
      }


      if (putCmt) outStream.println("| Describes the default capacistance.");
      if (nonEmpty(data.ibsCpkgtyp))
      {
         outStream.println("[C_pkg] " 
                            + " " + getVal(data.ibsCpkgtyp)
                            + " " + getVal(data.ibsCpkgmin)
                            + " " + getVal(data.ibsCpkgmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[C_pkg] typ min max" );
      }


      if (putCmt) outStream.println("| Describes the silicon die capacitance.");
      if (nonEmpty(data.ibsCcompCtyp))
      {
         outStream.println("[C_comp] "  
                            + " " + getVal(data.ibsCcompCtyp)
                            + " " + getVal(data.ibsCcompCmin)
                            + " " + getVal(data.ibsCcompCmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[C_comp] typ min max" );
      }


      if (putCmt)
      {
         outStream.println("| Describes the load resistance to use when performing the");
         outStream.println("| simulations for the ramp rate data.");
      }
      if (nonEmpty(data.ibsRload))
      {
         outStream.println("[Rload]  " + getVal(data.ibsRload));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[Rload] " );
      }


      if (putCmt) outStream.println( "|  Describes the transient simulation time to be used by Spice." );
      if (nonEmpty(data.ibsSimTime))
      {
         outStream.println( "[Sim time] " + " " + getVal(data.ibsSimTime));
         outStream.println("");
      }
      else
      {  
         if (putCmt) outStream.println( "|[Sim time] ");
      }


      if (putCmt)
      {
         outStream.println( "|   Default: [Pulldown reference] voltage if defined; 0 otherwise.");
         outStream.println( "|   Describes the low stimulus input voltage. Note that this is NOT");
         outStream.println( "|   the logic low voltage, but the physical Vil.");
      }
      if(nonEmpty(data.ibsVilVtyp))
      {
         outStream.println("[Vil] "
                            + " " + getVal(data.ibsVilVtyp)
                            + " " + getVal(data.ibsVilVmin)
                            + " " + getVal(data.ibsVilVmax));
         outStream.println("");
      }
      else
      {  
         if (putCmt) outStream.println( "|[Vil] typ min max" );
      }

      if (putCmt) 
      {
         outStream.println( "|   Default: [Pullup reference] voltage if defined; [Voltage range]");
         outStream.println( "|            voltage otherwise." );
         outStream.println( "|   Describes the high stimulus input voltage. Note that this is NOT" );
         outStream.println( "|   the logic high voltage, but the physical Vih." );
      }
      if (nonEmpty(data.ibsVihVtyp))
      {
         outStream.println("[Vih] "
                           + " " + getVal(data.ibsVihVtyp)
                           + " " + getVal(data.ibsVihVmin)
                           + " " + getVal(data.ibsVihVmax));
         outStream.println("");
      }
      else
      {  
         if (putCmt) outStream.println("|[Vih] typ min max");
      }

      if (putCmt) outStream.println("|   Describes the stimulus input voltage risetime.");
      if (nonEmpty(data.ibsTrTtyp))
      {
         outStream.println("[Tr] "
                           + " " + getVal(data.ibsTrTtyp)
                           + " " + getVal(data.ibsTrTmin)
                           + " " + getVal(data.ibsTrTmax));
         outStream.println("");
      }
      else
      {  
         if (putCmt) outStream.println("|[Tr] typ min max");
      }


      if (putCmt) outStream.println("|   Describes the stimulus input voltage falltime.");
      if (nonEmpty(data.ibsTfTtyp))
      {
         outStream.println("[Tf] "
                            + " " + getVal(data.ibsTfTtyp)
                            + " " + getVal(data.ibsTfTmin)
                            + " " + getVal(data.ibsTfTmax));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Tf] typ min max");
      }


      if (putCmt) 
      {
         outStream.println("|   Describes the threshold for printing lines in the clamp curves.");                      
         outStream.println("|   s2ibis2 suppresses printing of current values whose absolute");
         outStream.println("|   value is below the threshold.");
      }
      if (nonEmpty(data.ibsClampTol))
      {
         outStream.println("[Clamp tolerance] "
                      + " " + getVal(data.ibsClampTol));
         outStream.println("");
      }
      else
      {  
         if (putCmt) outStream.println("|[Clamp tolerance]");
      }
 

      if (putCmt)
      {
         outStream.println("|   Describes the percentage to derate the VI curves, as described");                       
         outStream.println("|   in the IBIS v2.1 spec.");
      }
      if (nonEmpty(data.ibsDerateVi))
      {
         outStream.println("[Derate VI] " + " " + getVal(data.ibsDerateVi) );
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Derate VI] ");
      }
 
      if (putCmt)
      {
         outStream.println("|   Describes the percentage to derate the ramp rates, as described");               
         outStream.println("|   in the IBIS v2.1 spec.");
      }
      if (nonEmpty(data.ibsDerateRamp))
      {
         outStream.println( "[Derate ramp] "
                      + " " + getVal(data.ibsDerateRamp) );
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Derate ramp]");
      }


      if (putCmt)
      {
         outStream.println( "| THE COMPONENT DESCRIPTION" );
         outStream.println( "| THE [Component] KEYWORD" );
         outStream.println( "|   Component name" );
      }
      outStream.println("[Component] " + " " + getVal(data.ibsComponent));
      outStream.println("");

      if (putCmt)
      {
         outStream.println( "| THE COMPONENT HEADER" );
         outStream.println( "|   Manufacturer's name" );
      }
      outStream.println("[Manufacturer] " + " " + getVal(data.ibsManufacturer));
      outStream.println("");

      if (putCmt) outStream.println( "|   Package name" );
      if (nonEmpty(data.ibsPackageName))
      {
         outStream.println("[Package model] " 
                           + " " + getVal(data.ibsPackageName));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println( "|[Package model] " );
      }
 
      if (putCmt) outStream.println( "|   Spice file" );
      outStream.println("[Spice file] " + " " + getVal(data.ibsSpiceFile));
      outStream.println("");

      
      if (putCmt)
      {
         outStream.println( "| THE PIN LIST" );
         outStream.println( "|   pin_name........Name of the pin. Must be 5 characters or less." );
         outStream.println( "|   spice_node......Node name in the spice file which corresponds to" );
         outStream.println( "|                   this pin." );
         outStream.println( "|   signal_name.....Name of the signal associated with this pin." );
         outStream.println( "|   model_name......Name of the driver/receiver/terminator model" );
         outStream.println( "|                   associated with this pin. The model name must" );
         outStream.println( "|                   match one described by the [Model] keyword (see");
         outStream.println( "|                   below), unless the model name is one of POWER," );
         outStream.println( "|                   GND or NC." );
         outStream.println( "|   input_pin.......Name of the pin which supplies the input signal");
         outStream.println( "|                   to the current pin. Must match the name of" );
         outStream.println( "|                   another pin in the pin list. This name is used" );
         outStream.println( "|                   in the Spice simulations to determine where to" );
         outStream.println( "|                   apply the input stimulus." );
         outStream.println( "|   enable_pin......Name of the pin which enables the current pin." );
         outStream.println( "|                   Must match the name of another pin in the pin" );
         outStream.println( "|                   list. This name is used in the Spice" );
         outStream.println( "|                   simulations to determine where to apply the" );
         outStream.println( "|                   enable signal." );
      }
      outStream.println( "[Pin]" );


      // Generate the pin-list
      if (nonEmpty(data.spice_node) && nonEmpty(data.model_name))
      {
         outStream.println( data.spice_node + " " + data.spice_node + " " 
                            + data.spice_node + " " + data.model_name); 
      }
      if (nonEmpty(data.input_pin))
      {
         outStream.print( "-> " + data.input_pin);
         if (nonEmpty(data.enable_pin))
         {
            outStream.print( " " + data.enable_pin);
         }
         outStream.println("");
         outStream.println(data.input_pin + " " + data.input_pin + " " 
                           + data.input_pin + " none");
         if (nonEmpty(data.enable_pin))
         {
            outStream.println(data.enable_pin + " " + data.enable_pin + " " 
                              + data.enable_pin + " none");
         }
      }
      if (nonEmpty(data.pow_pin))
      {
         outStream.println( data.pow_pin + " " + data.pow_pin + " " 
                            + data.pow_pin + " POWER" ); 
      }
      if (nonEmpty(data.gnd_pin))
      {
         outStream.println( data.gnd_pin + " " + data.gnd_pin + " " 
                            + data.gnd_pin + " GND" ); 
      }
      outStream.println("");

      /*  N O   L I S T S   N O W
      for( int i=0; i<data.ibsPinList.getItemCount(); ++i)
      {
         String temp = new String( "" );
         StringTokenizer st = new StringTokenizer( data.ibsPinList.getItem(i) );
         int tokens = st.countTokens();
         for(int j=0; j<tokens; ++j)
         {
            // s2iError.Debug( "StringToken " + i + " is " + temp );
            String temp2 = st.nextToken();
            if(j==0)
            {
               temp = temp2;
            }
            if(j==1)
            {
               temp = temp.concat( " " );
               temp = temp.concat( temp2 );
            }
            if(j==2)
            {
               temp = temp.concat( " " );
               temp = temp.concat( temp2 );
            }
            if(j==3)
            {
               temp = temp.concat( " " );
               temp = temp.concat( temp2 );
               model_name = temp2.toString();
            }
            if(j==4)
            {
               temp = temp.concat( "\n" );
               temp = temp.concat( "-> " );
               temp = temp.concat( temp2 );
               input_pin = temp2.toString();
            }
            if(j==5)
            {
               temp = temp.concat( " " );
               temp = temp.concat( temp2 );
               enable_pin = temp2.toString()
            }
         }
         outStream.println( temp );
      }
      outStream.println("");
      N O   L I S T S   N O W */


      if (putCmt)
      {
         outStream.println("| THE MODEL SPECIFICATION ");
         outStream.println("|   Model name" );
      }
      
      /* Models information starts
      if ((data.ibsNoModel != null) && (data.ibsNoModel.equals("Yes")))
      {
         outStream.println("[NoModel] ");
         outStream.println("");
      }
      */

      if (nonEmpty(data.model_name))
      {
         outStream.println("[Model]  " + data.model_name);
      }
      if (putCmt)
      {
         outStream.println("|   Possible types :");
         outStream.println("|       Input, Output, I/O, 3-State, Open_drain,");
         outStream.println("|       I/O_open_drain, Open_sink, I/O_open_sink,");
         outStream.println("|       Open_source, I/O_open_source, Input_ECL,");
         outStream.println("|       Output_ECL, I/O_ECL or Terminator");
      }

      outStream.println("[Model type] " + " " + getVal(data.ibsModelType));
      outStream.println("");

      if (putCmt)
      {
         outStream.println("|   Defines the model polarity.");
         outStream.println("|   Possible values : Inverting or Non-Inverting");
      }
      if (nonEmpty(data.ibsPolarity))
      {
         outStream.println("[Polarity] " + " " + getVal(data.ibsPolarity) );
         outStream.println( "" );
      }

      if (putCmt)
      {
         outStream.println("|   Defines how the model is enabled.");
         outStream.println("|   enable = Active-High or Active-Low");
      }
      if (nonEmpty(data.ibsEnable))
      {
         outStream.println("[Enable] " + " " + getVal(data.ibsEnable));
         outStream.println("");
      }

      if (putCmt)
      {
         outStream.println("|   Default: 0.8V for non-ECL, -1.475V for ECL");
         outStream.println("|   Defines the low input threshold voltage.");
      }
      if (data.ibsVin != null)
      {
          outStream.println("[Vinl] " + " " + (data.ibsVin));
          outStream.println("");
      }

      if (putCmt)
      {
         outStream.println("|   Default: 2.0V for non-ECL, -1.165V for ECL");
         outStream.println("|   Defines the high input threshold voltage.");
      }
      if (data.ibsVinh != null)
      {
          outStream.println("[Vinh] " + " " + (data.ibsVinh));
          outStream.println("");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the reference voltage level for board-level timing");
         outStream.println("|   simulation.");
      }
      if (nonEmpty(data.ibsVmeas))
      {
         outStream.println("[Vmeas] " + " " + getVal(data.ibsVmeas));
         outStream.println("");
      }
      else
      {
         if (putCmt) outStream.println("|[Vmeas] ");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the capacitive load used when specifying the propagation");
         outStream.println("|   delay or output switching time.");
      }
      if (nonEmpty(data.ibsCref))
      {
         outStream.println("[Cref] " + " " + getVal(data.ibsCref));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Cref] ");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the resistive load used when specifying the propagation");
         outStream.println("|   delay or output switching time.");
      }
      if (nonEmpty(data.ibsRref))
      {
         outStream.println("[Rref] " + " " + getVal(data.ibsRref));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Rref] ");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the load voltage used when specifying the propagation");
         outStream.println("|   delay or output switching time.");
      }
      if (nonEmpty(data.ibsVref))
      {
         outStream.println("[Vref] " + " " + getVal(data.ibsVref));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Vref] ");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the terminator ground resistance. Only valid for models");
         outStream.println("|   of type Terminator.");
      }
      if (nonEmpty(data.ibsRgndRtyp))
      {
         outStream.println("[Rgnd] "
                            + " " + getVal(data.ibsRgndRtyp)
                            + " " + getVal(data.ibsRgndRmin)
                            + " " + getVal(data.ibsRgndRmax));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Rgnd] typ min max");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the terminator power resistance. Only valid for models");
         outStream.println("|   of type Terminator.");
      }
      if (nonEmpty(data.ibsRpowerRtyp))
      {
         outStream.println("[Rpower] "
                            + " " + getVal(data.ibsRpowerRtyp)
                            + " " + getVal(data.ibsRpowerRmin)
                            + " " + getVal(data.ibsRpowerRmax));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Rpower] typ min max");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the terminator RC resistance. Only valid for models of");
         outStream.println("|   type Terminator.");
      }
      if (nonEmpty(data.ibsRacRtyp))
      {
         outStream.println("[Rac] "
                            + " " + getVal(data.ibsRacRtyp)
                            + " " + getVal(data.ibsRacRmin)
                            + " " + getVal(data.ibsRacRmax));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Rac] typ min max");
      }

      if (putCmt)
      {
         outStream.println("|   Defines the terminator RC capacitance. Only valid for models of");
         outStream.println("|   type Terminator.");
      }
      if (nonEmpty(data.ibsCacCtyp))
      {
         outStream.println("[Cac] "
                            + " " + getVal(data.ibsCacCtyp)
                            + " " + getVal(data.ibsCacCmin)
                            + " " + getVal(data.ibsCacCmax));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Cac] typ min max");
      }

      if (putCmt)
      {
         outStream.println("|   Specifies model files to be used for Spice simulations. Note");
         outStream.println("|   that the model files may include any valid Spice constructs--");
         outStream.println("|   s2ibis2 simply copies the named file into the Spice input file");
         outStream.println("|   before calling Spice.");
      }
      if (nonEmpty(data.ibsModelFiletyp))
      {
         outStream.println("[Model file] "
                            + " " + getVal(data.ibsModelFiletyp)
                            + " " + getVal(data.ibsModelFilemin)
                            + " " + getVal(data.ibsModelFilemax));
         outStream.println("");
      }
      else 
      {
         if (putCmt) outStream.println("|[Model file] ");
      }

      if (putCmt)
      {
         outStream.println("|   These keywords specify that s2ibis2 is to generate a rising or");
         outStream.println("|   falling waveform using the parameters listed. Note that _all_");
         outStream.println("|   columns must be filled, although only R_f and V_f_typ must be");
         outStream.println("|   specified. Please use the reserved word NA to fill a column");
         outStream.println("|   whose value you do not wish to specify. You may have up to 100");
         outStream.println("|   rising waveforms and 100 falling waveforms per model.");
         outStream.println("|   Parameters : R_f  V_f_typ  V_f_min  V_f_max  L_f  C_f  R_d  C_d L_d");
         outStream.println("|        R_f =     test fixture resistance");
         outStream.println("|        V_f_typ =     test fixture load voltage");
         outStream.println("|        V_f_min = test fixture load voltage for MIN curve");
         outStream.println("|        V_f_max = test fixture load voltage for MAX curve");
         outStream.println("|        L_f =     test fixture inductance");
         outStream.println("|        C_f =     test fixture capacitance");
         outStream.println("|        R_d =     package parasitic resistance");
         outStream.println("|        C_d =     package parasitic capacitance");
         outStream.println("|        L_d =     package parasitic inductance");
      }
      for (int i = 0; ( (i < data.RwaveCtr) || (i < data.FwaveCtr) ); i++)
      {
         if ( (i < data.RwaveCtr) && nonEmpty(data.ibsRwaveRf[i]))
         {
            outStream.println("[Rising waveform] "
                               + " " + getVal(data.ibsRwaveRf[i])
                               + " " + getVal(data.ibsRwaveVf[i])
                               + " " + getVal(data.ibsRwaveVfMin[i])
                               + " " + getVal(data.ibsRwaveVfMax[i])
                               + " " + getVal(data.ibsRwaveLf[i])
                               + " " + getVal(data.ibsRwaveCf[i])
                               + " " + getVal(data.ibsRwaveRd[i])
                               + " " + getVal(data.ibsRwaveCd[i])
                               + " " + getVal(data.ibsRwaveLd[i]));
            outStream.println("");
         }
         else 
         {
            if (putCmt) outStream.println("|[Rising waveform] ");
         }
         if ( (i < data.FwaveCtr) && nonEmpty(data.ibsFwaveRf[i]))
         {
            outStream.println("[Falling waveform] "
                               + " " + getVal(data.ibsFwaveRf[i])
                               + " " + getVal(data.ibsFwaveVf[i])
                               + " " + getVal(data.ibsFwaveVfMin[i])
                               + " " + getVal(data.ibsFwaveVfMax[i])
                               + " " + getVal(data.ibsFwaveLf[i])
                               + " " + getVal(data.ibsFwaveCf[i])
                               + " " + getVal(data.ibsFwaveRd[i])
                               + " " + getVal(data.ibsFwaveCd[i])
                               + " " + getVal(data.ibsFwaveLd[i]));
            outStream.println("");
         }
         else 
         {
            if (putCmt) outStream.println("|[Falling waveform]");
         }
      }

      
      /*As of now, we are only supporting one model.
        Hence the following the prints can be hard-coded.
      */
      outStream.println("[Model]  none");
      outStream.println("[NoModel] ");
      outStream.println("");


      outStream.flush();
      outStream.close();
   }


   public static void main(String args[])
   {
      s2iData data = new s2iData();
      s2iReader Rdr = new s2iReader();
      Rdr.Sp2iParse(args[0], data);

      s2iWriter Wtr = new s2iWriter();
      try
      {
         Wtr.outputS2i(args[1], data, false);
      }
      catch (java.io.IOException ioe)
      {
         String Msg = ioe.getMessage();
         s2iError.PrintMsg(Msg, s2iError.ERR,
                            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                            null);
      }
   }
}
