@echo off

REM This is the startup script for SpiTran on Windows.
REM See the usage below for more info. You may customize
REM this script to use s2ibis2 by full path, use a
REM different IBIS file viewer, or turn on debugging,
REM for example.

REM Check the arguments.
if "%1" == "-root" goto cdRoot
:dirset
if "%1" == "-help" goto showhelp
if "%1" == "-s2ibis" goto s2ibis
if not "%1" == "" goto usage:

REM Check the installation. At this point the script
REM should be running in the SpiTran installation root.
if not exist "data" goto dirmissing
if not exist "java" goto dirmissing

REM Set the incremental CLASSPATH.
set cp="java"

REM Run the SpiTran GUI and check for errors.
REM To enable debugging add "-debug" after s2iMain below.
REM The Java code MUST be run in the SpiTran root directory.
title SpiTran console
echo Running Java
java -cp %cp% s2iMain
if errorlevel 1000 jre -cp %cp% s2iMain
if errorlevel 1000 goto javamissing
if errorlevel 1 goto javaerror
exit 0

:dirmissing
echo.
echo Installation error.
echo This script must be run in the SpiTran installation directory.
echo You might try using the -root option to tell SpiTran where the
echo installation directory is.
echo.
pause
exit 1

:javaerror
echo errorlevel = %errorlevel%
java -version
echo.
echo An error occured running the Java program.
echo If the Java version above is less than 1.2, please install Java 2.
echo.
pause
exit 1

:javamissing
echo.
echo Java is required, but not installed. Please install Java 2.
echo.
pause
exit 1

REM Show the User Guide in the default web browser.
:showhelp
shift
cd doc\userguide
rundll32 url.dll,FileProtocolHandler index.html
exit

REM Run s2ibis2 and view the resulting IBIS file. The parameters are:
REM 1 the project directory.
REM 2 the s2i file in the project directory.
REM 3 the IBIS file that will be produced.
:s2ibis
shift
cd %1
s2ibis2 %2
if exist %3 write %3
exit

REM Change to the specified root directory and return to arg processing.
:cdRoot
shift
cd "%1"
shift
goto dirset

REM Display usage info for the user.
:usage
echo To start the SpiTran GUI:
echo spitran.bat
echo.
echo To show the SpiTran User Guide in a web browser:
echo spitran.bat [-root rootDir] -help
echo.
echo To run s2ibis2 (this is used by SpiTran):
echo spitran.bat [-root rootDir] -s2ibis projDir s2iFile ibisFile
echo.
echo The -root option may be needed to tell SpiTran where it's files are
exit 0
