#!/bin/csh -f

# This is the startup script for SpiTran on Unix.
# See the usage below for more info. You may customize
# this script to use s2ibis2 by full path, use a
# different IBIS file viewer, or turn on debugging,
# for example.

# Set the default browser and editor
if(! $?BROWSER) setenv BROWSER "netscape"
if(! $?PAGER) setenv PAGER "more"
if(! $?IBISVIEWER) setenv IBISVIEWER "xterm -e $PAGER"

# The location of the help file
set HELPURL = "file:`pwd`/doc/userguide/index.html"

@ i = 1
while($i <= $#argv)
  switch($argv[$i])
    case "-root":
      @ i = $i + 1
      set rootdir = "$argv[$i]"
      cd "$rootdir"
      breaksw
    case "-help":
      goto showhelp
      breaksw
    case "-s2ibis":
      @ i = $i + 1
      set projdir = "$argv[$i]"
      @ i = $i + 1
      set s2ifile = "$argv[$i]"
      @ i = $i + 1
      set ibisfile = "$argv[$i]"
      goto runs2ibis
      breaksw
    default:
      goto usage
      breaksw
  endsw
  @ i = $i + 1
end

# Getting ready to run SpiTran - search for a Java interpreter
if (! -d data || ! -d java) goto dirmissing
set javaprog = "`which java`"
if(! -x "$javaprog") set javaprog = "`which jre`"
if(! -x "$javaprog") goto javamissing

# Check the Java version - we require at least 1.2
$javaprog -version |& grep "java version.*1\.2" || goto javaversion

# Run the SpiTran GUI
# Add "-debug" after s2iMain below to turn on debugging
echo "Running Java"
$javaprog -cp java s2iMain
set retval = $status
if ($retval != 0) goto javaerror
exit 0

showhelp:
switch($BROWSER)
  case "netscape":
    $BROWSER -remote "openURL($HELPURL)" || $BROWSER $HELPURL
    breaksw
  default:
    $BROWSER $HELPURL
    breaksw
endsw
exit $status

runs2ibis:
cd $projdir
s2ibis2 $s2ifile |& tee s2ibis2.log
if(-f $ibisfile) $IBISVIEWER $ibisfile
exit 0

dirmissing:
echo ""
echo "Installation error."
echo "This script must be run in the SpiTran installation directory."
echo ""
exit 1

javaerror:
java -version
echo ""
echo "An error occured running the Java program."
echo "If the Java version above is less than 1.2, please install Java 2."
echo ""
exit 2

javamissing:
echo ""
echo "Java is required, but not installed. Please install Java 2."
echo ""
exit 3

javaversion:
java -version
echo ""
echo "The installed Java is not version 1.2.X. Please install Java 2."
exit 4

usage:
echo "To start the SpiTran GUI:"
echo "spitran.csh"
echo ""
echo "To show the SpiTran User Guide in a web browser:"
echo "spitran.csh [-root <rootDir>] -help"
echo ""
echo "To run s2ibis2 (this is used by SpiTran):"
echo "spitran.csh [-root <rootDir>] -s2ibis <projDir> <s2iFile> <ibisFile>"
echo ""
echo "The -root option may be needed to tell SpiTran where it's files are"
exit 0
